unit MainUnit; 

{$mode objfpc}{$H+}

{ ------------------------------------------------------------------------------
-
-             PortMidi bindings for FreePascal - a Basic Test Program
-
-                        latest changes 2017-12-01
------------------------------------------------------------------------------ }

interface

uses
  Interfaces,
  LResources,
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, CTypes,
  PortMidi,  // using PortMidi here  links the portaudio DLL on BuildTime
  PortTime; // for PtTimestamp


const
  ENABLE  = TRUE;
  DISABLE = FALSE;


type
  { TForm1 }
  TForm1 = class(TForm)
    ButtonClearMemo: TButton;
    ButtonClose: TButton;
    ButtonGetInfo: TButton;
    ButtonStart: TButton;
    ButtonStop: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    Label1: TLabel;
    Memo1: TMemo;
    Panel1: TPanel;
    Panel2: TPanel;
    Timer1: TTimer;
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure ButtonClearMemoClick(Sender: TObject);
    procedure ButtonGetInfoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { private declarations }
    procedure EnableObjects(b: boolean);
    procedure PtTestCallback( timestamp : PtTimestamp; userData : Pointer); cdecl;
  public
    { public declarations }
    TheError : PtError;
  end;

var
  Form1  : TForm1;
  aTime  : CInt32;
  aDummy : CInt32;


implementation

{ ***** TForm1 Data ********************************************************** }
procedure TForm1.PtTestCallback( timestamp : PtTimestamp; userData : Pointer); cdecl;
begin
  aTime:= timestamp;
  if userData <> NIL then aDummy:= Pt_Time;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Edit1.Text:= IntToStr( aTime);
  Edit2.Text:= IntToStr( aDummy);
end;

{ ***** TForm1 *************************************************************** }
procedure TForm1.FormCreate(Sender: TObject);
begin
  EnableObjects( ENABLE);
end;

procedure TForm1.ButtonCloseClick(Sender: TObject);
begin
  close;
end;

procedure TForm1.ButtonStartClick(Sender: TObject);
begin
  TheError := Pt_Start( 10, @PtTestCallback, nil);
  Memo1.Append( 'Pt_Start:' +#9 +'error = ' +IntToStr( TheError));
  EnableObjects( DISABLE);
end;

procedure TForm1.ButtonStopClick(Sender: TObject);
begin
  EnableObjects( ENABLE);

  TheError := Pt_Stop();
  Memo1.Append( 'Pt_Stop:' +#9 +'error = ' +IntToStr( TheError));
end;

procedure TForm1.ButtonClearMemoClick(Sender: TObject);
begin
  Memo1.Clear;
end;

procedure TForm1.ButtonGetInfoClick(Sender: TObject);
var Info : PPmDeviceInfo;
    i    : integer;
begin
  Memo1.Visible:= FALSE;
  Memo1.Clear;
  //Memo1.Append( 'Num Devices = '   +#9  +IntToStr( Pm_CountDevices));
  //Memo1.Append( 'DefaultInput = '  +#9  +IntToStr( Pm_GetDefaultInputDeviceID));
  //Memo1.Append( 'DefaultOutput = ' +#9  +IntToStr( Pm_GetDefaultOutputDeviceID));

  { enum devices : }
  for i:= 0 to  Pm_CountDevices -1 do
    begin
      Info:= Pm_GetDeviceInfo( i);
      Memo1.Append( 'Device ' +IntToStr(i) +': Interface ' +#9 +Info^.interf);
      if i = Pm_GetDefaultOutputDeviceID
        then Memo1.Append( 'Name '      +#9#9  +Info^.name
                           +'   ( DefaultOutput )')
        else
      if i = Pm_GetDefaultInputDeviceID
        then Memo1.Append( 'Name '      +#9#9  +Info^.name
                           +'   ( DefaultInput )')
        else Memo1.Append( 'Name '      +#9#9  +Info^.name);
      if Info^.input > 0
        then Memo1.Append( 'Input '     +#9#9  +IntToStr( Info^.input));
      if Info^.output > 0
        then Memo1.Append( 'Output '    +#9#9  +IntToStr( Info^.output));
      Memo1.Append( '');
    end;
  Memo1.Visible:= TRUE;
end;

procedure TForm1.EnableObjects(b: boolean);
begin
  if b
    then
      begin
        Edit1.Text:= '';
        Edit2.Text:= '';
      end

    else
      begin
        //
      end;

  Timer1.Enabled:= not( b);
  ButtonStart.Enabled:= b;
  ButtonStop.Enabled:= not( b);
  ButtonClearMemo.Enabled:= b;
end;


initialization
  {$I mainunit.lrs}

end.

