unit UnitTestPArecord;

{$ifdef FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  {$ifdef FPC}
  Interfaces,
  Forms,
  StdCtrls,
  cTypes,
  {$ENDIF}
  Classes, SysUtils, Dialogs,
  portaudio;


{** @file patest_record.c
	@ingroup test_src
	@brief Record input into an array; Save array to a file; Playback recorded data.
	@author Phil Burk  http://www.softsynth.com
*}
{*
 * $Id: patest_record.c 1368 2008-03-01 00:38:27Z rossb $
 *
 * This program uses the PortAudio Portable Audio Library.
 * For more information see: http://www.portaudio.com
 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *}

{*
 * The text above constitutes the entire PortAudio license; however,
 * the PortAudio community also makes the following non-binding requests:
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version. It is also
 * requested that these non-binding requests be included along with the
 * license above.
 *}

type
  TFloatArray = array of Single;
  paTestData = Record
        frameIndex      : Cardinal;    //* Index into sample array. */
        maxFrameIndex   : Cardinal;
        recordedSamples : TFloatArray;
    end;


// You can test failure by opening with this value. */
// const SAMPLE_RATE = 17932;

const
  SAMPLE_RATE       = 44100;
  FRAMES_PER_BUFFER =  1024;
  NUM_SECONDS       =     5;
  NUM_CHANNELS      =     2;

  //* Select sample format. */
  PA_SAMPLE_TYPE  = paFloat32;
  SAMPLE_SILENCE  = 0.0;


{* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().             *}
function recordCallback( const inputBuffer   : Pointer;
                         {%H-}outputBuffer   : Pointer;
                         framesPerBuffer     : Cardinal;
                         const {%H-}timeInfo : PPaStreamCallbackTimeInfo;
                         {%H-}statusFlags    : TPaStreamCallbackFlags;
                         userData            : Pointer) : CInt32; cdecl;

{* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*}
function playCallback( const {%H-}inputBuffer : Pointer;
                       outputBuffer           : Pointer;
                       framesPerBuffer        : Cardinal;
                       const {%H-}timeInfo    : PPaStreamCallbackTimeInfo;
                       {%H-}statusFlags       : TPaStreamCallbackFlags;
                       userData               : Pointer ) : Integer; cdecl;

function TestMain() : Integer;



{$ifdef FPC}
{  replace WriteLn() for GUI usage :  }
procedure SetMemo( Memo : TMemo);
function writeln( s : string) : integer;
{$ENDIF}


implementation


{$ifdef FPC}
var InfoMemo : TMemo;

procedure SetMemo( Memo : TMemo);
begin
  InfoMemo:= Memo;
end;

function writeln( s : string) : integer;
begin
  Result:= 0;
  InfoMemo.Append( s);
end;
{$ENDIF}


{* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().             *}
function recordCallback( const inputBuffer : Pointer;
                         outputBuffer      : Pointer;
                         framesPerBuffer   : Cardinal;
                         const timeInfo    : PPaStreamCallbackTimeInfo;
                         statusFlags       : TPaStreamCallbackFlags;
                         userData          : Pointer) : CInt32; cdecl;
var data         : ^paTestData;
    rPtr         : ^Single;
    wPtr         : ^Single;
    framesToCalc : Cardinal;
    i            : Integer;
    finished     : Integer;
    framesLeft   : Cardinal;
begin
  data:= @userData^;
  rPtr:= @inputBuffer^;
  wPtr:= @data^.recordedSamples[ data^.frameIndex * NUM_CHANNELS];
  framesLeft:= data^.maxFrameIndex - data^.frameIndex;

  if framesLeft < framesPerBuffer
    then
      begin
        framesToCalc:= framesLeft;
        finished:= paComplete;
      end
    else
      begin
        framesToCalc:= framesPerBuffer;
        finished:= paContinue;
      end;

  if inputBuffer = NIL
    then
      for i:= 0 to framesToCalc - 1 do
        begin
          wptr^:= SAMPLE_SILENCE;  //* left */
          inc( wPtr);
          if NUM_CHANNELS = 2 then
            begin
              wPtr^:= SAMPLE_SILENCE;  //* right */
              inc( wPtr);
            end;
      end

    else
      for i:= 0 to framesToCalc - 1 do
        begin
          wPtr^:= rPtr^;
          inc( wPtr);
          inc( rPtr);
          if NUM_CHANNELS = 2 then
            begin
              wPtr^:= rPtr^;
              inc( wPtr);
              inc( rPtr);
            end;
        end;

  data^.frameIndex:= data^.frameIndex + framesToCalc;
  Result:= finished;
end;


{* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*}
function playCallback( const inputBuffer : Pointer;
                       outputBuffer      : Pointer;
                       framesPerBuffer   : Cardinal;
                       const timeInfo    : PPaStreamCallbackTimeInfo;
                       statusFlags       : TPaStreamCallbackFlags;
                       userData          : Pointer ) : Integer; cdecl;
var
  data  : ^paTestData;
  rPtr  : ^Single;
  wPtr  : ^Single;
  i     : Cardinal;
  finished   : Integer;
  framesLeft : Cardinal;
begin
  data:= userData;
  rPtr:= @data^.recordedSamples[ data^.frameIndex * NUM_CHANNELS];
  wPtr:= outputBuffer;
  framesLeft:= data^.maxFrameIndex - data^.frameIndex;

  TRY
  if framesLeft < framesPerBuffer
    then
      begin
        //* final buffer... */
        for i:= 0 to framesLeft - 1 do
          begin
            wPtr^:= rPtr^;
            Inc( wPtr);
            Inc( rPtr);
            if( NUM_CHANNELS = 2 )then
            begin
              wPtr^:= rptr^;
              Inc( wPtr);
              Inc( rPtr);
            end;
          end;

        { take over "i" from prevoius loop : }
        while i < framesPerBuffer do
          begin
            wPtr^:= 0;
            inc( wPtr);
            if NUM_CHANNELS = 2 then
              begin
                wptr^:= 0;
                inc( wPtr);  //* right */
              end;
            inc( i);
          end;
          data^.frameIndex := data^.frameIndex + framesLeft;
          finished := paComplete;
      end   // .. if framesLeft < framesPerBuffer

    else
      begin
        for i:= 0 to framesPerBuffer - 1 do
          begin
            wPtr^ := rPtr^;
            inc( wPtr);
            inc( rPtr);
            if NUM_CHANNELS = 2 then
            begin
              wPtr^:= rPtr^;
              inc( wPtr);
              inc( rPtr);
            end;
          end;
        data^.frameIndex:= data^.frameIndex + framesPerBuffer;
        finished:= paContinue;
    end;
  EXCEPT
    ShowMessage( 'Exception in playCallback()');
  end;

  Result:= finished;
end;


//*******************************************************************/
//*****  the main function  *****************************************/
//*******************************************************************/
function TestMain() : Integer;
var
  inputParameters  : TPaStreamParameters;
  outputParameters : TPaStreamParameters;
  pStream     : PPaStream;
  err         : TPaError;
  data        : paTestData;
  i           : Integer;
  totalFrames : Integer;
  numSamples  : Integer;
  max, val    : Single;
  average     : Double;
Label done;
begin
  err:= paNoError;
  writeln( 'patest_record.c');
  writeln( '');

  data.maxFrameIndex := NUM_SECONDS * SAMPLE_RATE; //* Record for a few seconds. */
  totalFrames        := NUM_SECONDS * SAMPLE_RATE; //* Record for a few seconds. */
  data.frameIndex    := 0;
  numSamples         := totalFrames * NUM_CHANNELS;

  SetLength( data.recordedSamples, numSamples); //* From now on, recordedSamples is initialised. */
  for i:= 0 to numSamples - 1 do data.recordedSamples[i]:= 0;

  err:= Pa_Initialize();
  if err <> paNoError then
    begin
      writeln( 'Could not initialize PortAudio.');
      exit;
    end;

  inputParameters.device:= Pa_GetDefaultInputDevice(); //* default input device */
  if inputParameters.device = paNoDevice then
    begin
      writeln( 'Error:  No default input device.');
      goto done;
    end;

  inputParameters.channelCount:= 2;                    //* stereo input */
  inputParameters.sampleFormat:= PA_SAMPLE_TYPE;
  average:= Pa_GetDeviceInfo( inputParameters.device )^.defaultLowInputLatency;
  inputParameters.suggestedLatency:= Pa_GetDeviceInfo( inputParameters.device )^.defaultLowInputLatency;
  inputParameters.hostApiSpecificStreamInfo:= NIL;

  Application.ProcessMessages; Sleep( 100);
  {$ifdef FPC} pStream:= NIL; {$ENDIF} // make compiler happy ..
  //* Record some audio. -------------------------------------------- */
  err:= Pa_OpenStream( pStream,
                       @inputParameters,
                       NIL,            //* &outputParameters, */
                       SAMPLE_RATE,
                       FRAMES_PER_BUFFER,
                       paClipOff,      //* we won't output out of range samples so don't bother clipping them */
                       {$ifdef FPC}
                       PPaStreamCallback( @recordCallback),
                       {$ELSE}
                       @recordCallback,
                       {$ENDIF}
                       @data );
  if err <> paNoError then
    begin
      writeln( 'Pa_OpenStream() Error: ' +IntToStr( err));
      goto done;
    end;

  err:= Pa_StartStream( pStream);
  if err <> paNoError then
    begin
      writeln( 'Pa_StartStream() Error: ' +IntToStr( err));
      goto done;
    end;
  writeln( 'Now recording !');

  { wait loop .. }
  repeat
    err:= Pa_IsStreamActive( pStream);
    if err <> 1 then break;
    Pa_Sleep( 1000);
    writeln( 'index = '+IntToStr( data.frameIndex) );
  until err <> 1;

  err:= Pa_CloseStream( pStream);
  if err <> paNoError then goto done;


  //* Measure maximum peak amplitude. */
  max:= 0;
  average:= 0.0;
  for i:= 0 to numSamples - 1 do
    begin
      val:= data.recordedSamples[i];
      if val < 0 then val:= -val; //* ABS */
      if val > max then max:= val;
      average:= average + val;
    end;

  average:= average / numSamples;

  writeln( 'sample max amplitude = ' + FormatFloat('0.00',max ));
  writeln( 'sample average = '+FormatFloat('0.00', average ));

  //* Playback recorded data.  -------------------------------------------- */
  data.frameIndex:= 0;

  outputParameters.device:= Pa_GetDefaultOutputDevice(); //* default output device */
  if outputParameters.device = paNoDevice then
    begin
      writeln( 'Error: No default output device.');
      goto done;
    end;
  outputParameters.channelCount:= 2;                     //* stereo output */
  outputParameters.sampleFormat:= PA_SAMPLE_TYPE;
  outputParameters.suggestedLatency:= Pa_GetDeviceInfo( outputParameters.device )^.defaultLowOutputLatency;
  outputParameters.hostApiSpecificStreamInfo:= NIL;

  writeln( '');
  writeln( 'Begin playback.');
  err:= Pa_OpenStream( pStream,
                       nil, //* no input */
                       @outputParameters,
                       SAMPLE_RATE,
                       FRAMES_PER_BUFFER,
                       paClipOff,      //* we won't output out of range samples so don't bother clipping them */
                       {$ifdef FPC}
                       PPaStreamCallback( @playCallback),
                       {$ELSE}
                       @playCallback,
                       {$ENDIF}
                       @data );
  if err <> paNoError then
    begin
      writeln( 'Pa_OpenStream() Error: ' +IntToStr( err));
      goto done;
    end;

  //* begin  playback
  err:= Pa_StartStream( pStream);
  if err <> paNoError then goto done;
  writeln( 'Waiting for playback to finish.');

  repeat
    err:= Pa_IsStreamActive( pStream);  // returns one (1) when the stream is active
    if err = 1
      then Pa_Sleep( 750)
      else Break;
    writeln( 'index = '+IntToStr( data.frameIndex) );
  until err <> 1;

  if err < 0 then
    begin
      writeln( 'Pa_IsStreamActive() Error: ' +IntToStr( err));
      goto done;
    end;

  err:= Pa_StopStream( pStream);
  if not (err = paNoError) then
    begin
      writeln( 'Pa_StopStream() Error: ' +IntToStr( err));
    end;

  Application.ProcessMessages; Sleep( 100); // added 2021-02-09
  repeat
    err:= Pa_IsStreamStopped( pStream);
  until err = 1;  //  paNoError would be wrong here !

  // With the current PortAudio.DLL
  // somehow this DemoExe crashes on Pa_CloseStream ..
{  TRY
    err:= Pa_CloseStream( pStream);
  EXCEPT
    writeln( 'Pa_CloseStream() - Exception !');
  end;
  if not (err = paNoError)
    then writeln( 'Pa_CloseStream() - Error: ' +IntToStr( err));
}

done:
// With the current PortAudio.DLL
// somehow this DemoExe crashes on Pa_CloseStream ..
{  TRY
    err:= Pa_Terminate();
  EXCEPT
    writeln( 'Pa_Terminate() Exception !');
  end;
  if not (err = paNoError)
    then writeln( 'Pa_Terminate() Error: ' +IntToStr( err));
}

  SetLength( data.recordedSamples, 0);

{  if( err <> paNoError )then
    begin
      writeln( 'An error occured while using the portaudio stream' );
      writeln( 'Error number: ' + IntToStr(err) );
      writeln( 'Error message: %s' + StrPas(Pa_GetErrorText( err )) );
      err:= 1;  //* Always return 0 or 1, but no other return codes. */
    end;
}
  Result:= 0; //err;
end;


end.

