unit MainUnitSine;

{$mode objfpc}{$H+}


{

  latest changes:  2019-04-16 - checked with Lazarus 2.0


  This is the LCL version of  FPC example program  TestPASine.pas
  from the HuMuS project.

  In August 2010 I switched to the PORTAUDIO.PAS from ULTRASTAR project
  because this source file is well documented and all needed code
  resides in just one file which makes life easier .. ;-)

  ---------------------------------------------------------------

  PortAudio bindings for FPC by the HuMuS team - Sine test program
  Latest version available at: http://sourceforge.net/apps/trac/humus/

  Copyright (c) 2009-2010 Roland Schaefer

  PortAudio Portable Real-Time Audio Library
  Latest version available at: http://www.portaudio.com

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  The text above constitutes the entire PortAudio license; however,
  the PortAudio community also makes the following non-binding requests:

  Any person wishing to distribute modifications to the Software is
  requested to send the modifications to the original developer so that
  they can be incorporated into the canonical version. It is also
  requested that these non-binding requests be included along with the
  license above.
}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ComCtrls, CTypes,
  { using unit PortAudio here  statically links the portaudio DLL ! }
  PortAudio;


const
  SampleRate = 44100;
  FramesPerBuffer = 64;

  // How long you want to play the test sine:
  NumSecs = 2;

  // Wavetable size. Influences your pitch:
  TableSize = 200;

type
  { A type which holds a wavetable, two integers keeping track of
    at which offset in the wavetable each channel is currently
    playing (= phase), and a message: }
  PaTestData = record
    Sine : array[0..TableSize] of CFloat;
    LeftPhase : CInt32;
    RightPhase : CInt32;
    AMessage : PChar;
  end;
  PPaTestData = ^PaTestData;


type

  { TForm1 }

  TForm1 = class(TForm)
    ButtonClose: TButton;
    Button2: TButton;
    ComboBox1: TComboBox;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    ProgressBar1: TProgressBar;
    procedure ButtonCloseClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var {%H-}CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    function ReportError( const err:TPaError; s:string):boolean;
    procedure ShowProgress;
//    procedure StreamFinished( UserData : pointer ); cdecl;
  public
    { public declarations }
    OutputParameters : TPaStreamParameters;
    { This pointer points to a pointer which will be returned by
      Pa_OpenStream: }
    Stream : PPaStream;
    Err : TPaError;
    Data : PaTestData;
    DataPointer : PPaTestData;
    j : CInt32;
    OutDevice : integer;
  end;

var
  Form1: TForm1; 


implementation


{ The callback function which is called by PA everytime new Data is needed.
  Remember: ALWAYS USE CDECL or your pointers will be messed up!
  Pointers to this function must be castable to PPaStreamCallback: }
function PaTestCallback( {%H-}inputBuffer : pointer;
                         OutputBuffer : pointer;
                         framesPerBuffer : culong;
                         {%H-}timeInfo : PPaStreamCallbackTimeInfo;
                         {%H-}statusFlags : TPaStreamCallbackFlags;
                         UserData : pointer ) : CInt32; cdecl;

var
  LocalDataPointer : PPaTestData;
  OutBuffer : PCFloat;
  i : culong;
begin
  OutBuffer := PCFloat(OutputBuffer);
  LocalDataPointer := PPaTestData(UserData);

  // Fill the buffer...
  for i := 0 to (FramesPerBuffer-1) do begin

    OutBuffer^ := LocalDataPointer^.Sine[LocalDataPointer^.LeftPhase];
    Inc(OutBuffer);

    OutBuffer^ := LocalDataPointer^.Sine[LocalDataPointer^.RightPhase];
    Inc(OutBuffer);

    Inc(LocalDataPointer^.LeftPhase);
    if (LocalDataPointer^.LeftPhase >= TableSize ) then
        LocalDataPointer^.LeftPhase :=
          (LocalDataPointer^.LeftPhase - TableSize);

    Inc(LocalDataPointer^.RightPhase);
    Inc(LocalDataPointer^.RightPhase);
    if ( LocalDataPointer^.RightPhase >= TableSize ) then
        LocalDataPointer^.RightPhase :=
          (LocalDataPointer^.RightPhase - TableSize);
  end;

  PaTestCallback:= CInt32( 0);
end;


{ This is called when playback is finished.
  Remember: ALWAYS USE CDECL or your pointers will be messed up!
  Pointers to this function must be castable to PPaStreamFinishedCallback: }
procedure StreamFinished( UserData : pointer ); cdecl;
//procedure TForm1.StreamFinished( UserData : pointer ); cdecl;
var
  {%H-}LocalDataPointer : PPaTestData;
begin
  LocalDataPointer := PPaTestData( UserData);
//  Edit1.Caption:= 'Stream Completed: ' +LocalDataPointer^.AMessage^;
end;



{ ***** TForm1 *************************************************************** }
procedure TForm1.FormCreate(Sender: TObject);
var
  pDEVinfo : PPaDeviceInfo;
  pAPIinfo : PPaHostApiInfo;
  sAPI : string;
  i : dword;
begin
  { Fill a Sine wavetable (Float Data -1 .. +1) }
  for i := 0 to TableSize - 1
    do Data.Sine[i]:= CFloat((Sin( ( CFloat(i)/CFloat(TableSize) ) * Pi * 2 )));

  Data.LeftPhase:= 0;
  Data.RightPhase:= 0;
  Data.AMessage:= 'No Message'#0;

  DataPointer:= @Data;

  Err:= Pa_Initialize;
  ReportError( Err, 'Pa_Initialize');

  //Combobox1.Caption:= 'DeviceCount = ' +IntToStr( Pa_GetDeviceCount);
  for i:= 0 to Pa_GetDeviceCount -1 do
    begin
      { get the pointer to the Info structures : }
      pDEVinfo:= Pa_GetDeviceInfo( i);
      pAPIinfo:= Pa_GetHostApiInfo( pDEVinfo^.hostApi);

      if pAPIinfo <> NIL then
        case pAPIinfo^._type of
          paDirectSound :  sAPI:= 'DirectSound';
          paMME         :  sAPI:= 'MME';
          paASIO        :  sAPI:= 'ASIO';
          paSoundManager:  sAPI:= 'SoundManager';
          paWDMKS       :  sAPI:= 'WDM-KS';
          paJACK        :  sAPI:= 'JACK';
          paWASAPI      :  sAPI:= 'WASAPI';
          else sAPI:= 'unknown';
        end;

      { only add outputs to the ComboBox : }
      if pDEVinfo <> NIL then
      if pDEVinfo^.maxOutputChannels > 0
        then Combobox1.AddItem( IntToStr(i) +' - ' +sAPI, NIL);
    end;
  Combobox1.ItemIndex:= 0;
end;


procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  // HINT:  with the PortAudio.dll from LINRAD project
  //        the test app chrashes (SIGSEGV) if You
  //        immediately press the CLOSE Button
  //        (without playing the SINE by pressing the TEST button)
  //          This happened on a Fujitsu Siemens Scenic W300
  //          with installed ASIOFORALL driver
  // No chrashes on TERMINATE occured with the other DLLs until now ..
  // Please see folder "PortAudio DLLs"
  Pa_Terminate();
end;


function TForm1.ReportError( const err:TPaError; s:string):boolean;
begin
  if Err = 0
    then
      begin
        Edit1.Text:= s +'  successful';
        result:= FALSE;
      end
    else
      begin  { an error occured}
        ShowMessage( s +' failed !');
        Edit1.Text:= s +' failed, Errorcode = ' +IntToStr( Err);
        result:= TRUE;
      end;
end;


procedure TForm1.ShowProgress;
begin
  ProgressBar1.Position:= ProgressBar1.Position +1;
  { let the surface update : }
  Application.ProcessMessages;

  { slow down everything, only to show the progress : }
  Sleep( 150);
end;


procedure TForm1.ButtonCloseClick(Sender: TObject);
begin
  close;
end;


procedure TForm1.Button2Click(Sender: TObject);
var a : dword;
begin
  ProgressBar1.Position:= 0;
  Button2.Enabled:= FALSE;

  Err:= Pa_Initialize;
  if ReportError( Err, 'Pa_Initialize') then Exit;

  Edit2.Caption:= IntToStr( SampleRate);
  Edit3.Caption:= IntToStr( FramesPerBuffer);
  ShowProgress;

  //on my system OK: 3, 4, 5 (but noSound),
  //                 9, 10, 11(but noSound), 12(ASIO), 13,
  OutDevice:= StrToInt(  trim( LeftStr( ComboBox1.Caption, 2)));
  OutputParameters.Device := OutDevice; //Pa_GetDefaultOutputDevice;
  Label7.Caption:= 'Output Device = ' +IntToStr( OutDevice); //Pa_GetDefaultOutputDevice);
  OutputParameters.ChannelCount := CInt32(2);
  OutputParameters.SampleFormat := paFloat32;
  OutputParameters.SuggestedLatency :=
    (Pa_GetDeviceInfo( OutputParameters.device)^.defaultHighOutputLatency) * 1;
  OutputParameters.HostApiSpecificStreamInfo := nil;

  Edit4.Caption:= FloatToStr(
    Pa_GetDeviceInfo( OutputParameters.device)^.defaultHighOutputLatency);

  Err := Pa_OpenStream( Stream, nil, @OutputParameters, SampleRate,
                        FramesPerBuffer, paClipOff,
                        PPaStreamCallback(@PaTestCallback), DataPointer);
  if ReportError( Err, 'Pa_OpenStream') then Exit;
  ShowProgress;


  Err := Pa_SetStreamFinishedCallback( Stream,
     PPaStreamFinishedCallback( @StreamFinished));
  if ReportError( Err, 'Pa_SetStreamFinishedCallback') then Exit;
  Edit5.Caption:= IntToStr( NumSecs);
  ShowProgress;


  Err := Pa_StartStream( Stream );
  if ReportError( Err, 'Pa_StartStream') then Exit;
  Edit6.Caption:= IntToStr( NumSecs);
  ShowProgress;

  for a:=1 to 5 do
    begin
      Pa_Sleep( NumSecs * 250 );
      ShowProgress;
    end;

  Err := Pa_StopStream( Stream );
  if ReportError( Err, 'Pa_StopStream') then Exit;
  ShowProgress;

  Err := Pa_CloseStream( Stream );
  if ReportError( Err, 'Pa_CloseStream') then Exit;
  ShowProgress;

  //Err := Pa_Terminate();
  //ReportError( Err, 'Pa_Terminate');


  //ShowMessage( 'Pa_CloseStream');
  Button2.Enabled:= TRUE;
  ShowProgress;
end;


initialization
  {$I mainunitsine.lrs}

end.

