unit PAwire;

{$mode objfpc}{$H+}

{*******************************************************************************
*                                                                              *
*  This project is based on the C example file  "patest_wire.c"                *
*  from PortAudio Project                                                      *
*                                                                              *
*                                                                              *
*  To reduce the size of the EXE file please use strip.bat                     *
*  which calls strip.exe in the main folder                                    *
*                                                                              *
*  Example Code converted to LAZARUS by BREAKOUTBOX in 09-2010                 *
*                                                                              *
********************************************************************************   
*                                                                              *
*  latest changes:  2021-02-09 - compiled with Lazarus 2.0.10                  *
*                                                                              *
********************************************************************************


Original text from the C file :
-------------------------------

   @file patest_wire.c
	        @ingroup test_src
	        @brief Pass input directly to output.

	        Note that some HW devices, for example many ISA audio cards
	        on PCs, do NOT support full duplex! For a PC, you normally need
	        a PCI based audio card such as the SBLive.

	        @author Phil Burk  http://www.softsynth.com

	 While adapting to V19-API, I excluded configs with framesPerCallback=0
	 because of an assert in file pa_common/pa_process.c. Pieter, Oct 9, 2003.

	 * This program uses the PortAudio Portable Audio Library.
	 * For more information see: http://www.portaudio.com
	 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
	 *
	 * Permission is hereby granted, free of charge, to any person obtaining
	 * a copy of this software and associated documentation files
	 * (the "Software"), to deal in the Software without restriction,
	 * including without limitation the rights to use, copy, modify, merge,
	 * publish, distribute, sublicense, and/or sell copies of the Software,
	 * and to permit persons to whom the Software is furnished to do so,
	 * subject to the following conditions:
	 *
	 * The above copyright notice and this permission notice shall be
	 * included in all copies or substantial portions of the Software.
	 *
	 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
	 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
	 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

	 * The text above constitutes the entire PortAudio license; however,
	 * the PortAudio community also makes the following non-binding requests:
	 *
	 * Any person wishing to distribute modifications to the Software is
	 * requested to send the modifications to the original developer so that
	 * they can be incorporated into the canonical version. It is also
	 * requested that these non-binding requests be included along with the
	 * license above.
*******************************************************************************}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, PortAudio, CTypes;



//{$define USE_FLOAT_INPUT}      //activate if You want to use cFloat
//{$define USE_FLOAT_OUTPUT}     //activate if You want to use cFloat

type INPUTFORMAT = TPaSampleFormat;
{$ifdef USE_FLOAT_INPUT}
var INPUT_FORMAT : INPUTFORMAT;
type INPUT_SAMPLE = cFloat;
{$else}
var INPUT_FORMAT : INPUTFORMAT;
type INPUT_SAMPLE = cShort;
{$endif}

type OUTPUTFORMAT = TPaSampleFormat;
{$ifdef USE_FLOAT_OUTPUT}
var OUTPUT_FORMAT : OUTPUTFORMAT;
type OUTPUT_SAMPLE = cFloat;
{$else}
var OUTPUT_FORMAT : OUTPUTFORMAT;
type OUTPUT_SAMPLE = cShort;
{$endif}

type TPointerList = array of pointer;


type
  { A type which holds a wavetable, two integers keeping track of
    at which offset in the wavetable each channel is currently
    playing (= phase), and a message: }
  PaTestData = record
    //Sine : array[0..TableSize] of CFloat;
    LeftPhase : CInt32;
    RightPhase : CInt32;
    //AMessage : PChar;
  end;
  PPaTestData = ^PaTestData;


var   gInOutScaler : cDouble;

const USE_FLOAT_INPUT  = 1;
      USE_FLOAT_OUTPUT = 1;
      SAMPLE_RATE      = 44100;  //FixMe:  fiext sample rate !


type
  PWireConfig = ^TWireConfig;
  TWireConfig = record
          isInputNotInterleaved  : boolean;  // Warning: in C language TRUE = 0
          isOutputNotInterleaved : boolean;  // in Lazarus TRUE = 1 !!!
          numInputChannels    : integer;
          numOutputChannels   : integer;
          framesPerCallback   : integer;
  end;


type
  { TForm1 }
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Memo1: TMemo;
    Panel1: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    stream : PPaStream;
    INPUT_DEVICE  : integer;
    OUTPUT_DEVICE : integer;
    INPUT_FORMAT  : integer;
    OUTPUT_FORMAT : integer;
    WireConfig    : TWireConfig;   // user data for callback
    function TestConfiguration( var config : TWireConfig):TPaError;
  public
    { public declarations }
  end; 


var
  Form1: TForm1; 


implementation

function CONVERT_IN_TO_OUT( val:INPUT_SAMPLE):OUTPUT_SAMPLE; inline;
begin
  {$ifdef USE_FLOAT_INPUT}
    {$ifdef USE_FLOAT_OUTPUT}
    result:= val;                         // both are floatingpoint
    {$else}
    result:= round( val * gInOutScaler);  // only input is floatingpoint
    {$endif}

  {$else}
    {$ifdef USE_FLOAT_OUTPUT}
    result:= val * gInOutScaler;          // only output is floatingpoint
    {$else}
    result:= val * round( gInOutScaler);  // both are NOT floatingpoint
    {$endif}                              // but gInOutScaler is floatingpoint

  {$endif}
end;

//** This routine will be called by the PortAudio engine when audio is needed.
//** It may be called at interrupt level on some machines so don't do anything
//** that could mess up the system like calling malloc() or free().
function wireCallback( inputBuffer     : pointer;
                       outputBuffer    : pointer;
	               framesPerBuffer : culong;
	               {%H-}timeInfo        : PPaStreamCallbackTimeInfo;
	               {%H-}statusFlags     : TPaStreamCallbackFlags;
	               userData        : pointer):CInt32; cdecl;
var pINPUT        : ^INPUT_SAMPLE;
    pOUTPUT       : ^OUTPUT_SAMPLE;
    inStride      : integer;
    outStride     : integer;
    inDone        : boolean;
    outDone       : boolean;
    pConfig       : PWireConfig;
    i             : dword;  //unsigned int
    inChannel     : integer;
    outChannel    : integer;
    pInList       : TPointerList;
    pOutList      : TPointerList;
begin
  inDone:=  FALSE;
  outDone:= FALSE;
  pConfig:= PWireConfig( @userData^);
  pInList:= NIL;   // make compiler happy ..
  pOutList:= NIL;  // make compiler happy ..
  setLength( pInList, pConfig^.numInputChannels);
  setLength( pOutList, pConfig^.numOutputChannels);

  // This may get called with NULL inputBuffer during initial setup.
  if( inputBuffer = NIL) then
    begin
      result:= 0;
      exit;
    end;

  inChannel:=  0;
  outChannel:= 0;

  while (not(inDone) or not(outDone)) do
    begin
      { Input }
      if pConfig^.isInputNotInterleaved = FALSE
        then
          begin  { interleaved }
            pINPUT:= @inputBuffer^;
            inc( pINPUT, inChannel);    // in case first channel:  inChannel = 0
            inStride:= pConfig^.numInputChannels;
          end
        else
          begin
            // The paNonInterleaved flag indicates that a multichannel buffer
            // is passed as a set of non-interleaved pointers.
            pInList:= @inputBuffer^;
            { set pointer to next block of NonInterleaved data : }
            pINPUT:= pInList[inChannel];
            inStride:= 1;
          end;

      { Output }
      if pConfig^.isOutputNotInterleaved = FALSE
        then
          begin  { interleaved }
            pOUTPUT:= @outputBuffer^;
            inc( pOUTPUT, outChannel);
            outStride:= pConfig^.numOutputChannels;
          end
        else
          begin
            pOutList:= @outputBuffer^;
            { set pointer to next block of NonInterleaved data : }
            pOUTPUT:= pOutList[outChannel];
            outStride:= 1;
          end;

      { pointers are set, now copy from IN to OUT buffer : }
      for i:= 0 to framesPerBuffer-1 do
        begin
          pOUTPUT^:= CONVERT_IN_TO_OUT( pINPUT^);
          inc( pINPUT, inStride);
          inc( pOUTPUT, outStride);
        end;

      { finally increment channel : }
      if inChannel < (pConfig^.numInputChannels - 1)
        then inc( inChannel)
        else inDone:= TRUE;

      if outChannel < (pConfig^.numOutputChannels - 1)
        then inc( outChannel)
        else outDone:= TRUE;
    end;

  setLength( pInList, 0);
  setLength( pOutList, 0);
  result:= 0;
end;


{ This is called when playback is finished.
  Remember: ALWAYS USE CDECL or your pointers will be messed up!
  Pointers to this function must be castable to PPaStreamFinishedCallback: }
procedure StreamFinished( UserData : pointer ); cdecl;
//procedure TForm1.StreamFinished( UserData : pointer ); cdecl;
var
  {%H-}LocalDataPointer : PPaTestData;
begin
  LocalDataPointer:= PPaTestData( UserData);
//  Edit1.Caption:= 'Stream Completed: ' +LocalDataPointer^.AMessage^;
end;


{ ***** TForm1 *************************************************************** }
procedure TForm1.FormCreate(Sender: TObject);
begin
  gInOutScaler:= 1;
  {$ifdef USE_FLOAT_INPUT}
  INPUT_FORMAT:= paFloat32;
  {$else}
  INPUT_FORMAT:= paInt16;
  {$endif}

  {$ifdef USE_FLOAT_OUTPUT}
  OUTPUT_FORMAT:= paFloat32;
  {$else}
  OUTPUT_FORMAT:= paInt16;
  {$endif}
end;


function TForm1.TestConfiguration( var config : TWireConfig):TPaError;
var inputParameters  : TPaStreamParameters;
    outputParameters : TPaStreamParameters;
    err : TPaError;
label error;
begin
  result:= 0;
  err:= paNoError;

  if config.isInputNotInterleaved
    then Memo1.Append( ' input:' +#9#9 +'NOT interleaved!')
    else Memo1.Append( ' input:' +#9#9 +'interleaved!');

  if config.isOutputNotInterleaved
    then Memo1.Append( ' output:' +#9#9 +'NOT interleaved!')
    else Memo1.Append( ' output:' +#9#9 +'interleaved!');

  Memo1.Append( ' input channels:' +#9 +IntToStr( config.numInputChannels));
  Memo1.Append( ' output channels:' +#9 +IntToStr( config.numOutputChannels));
  Memo1.Append( 'framesPerCallback:' +#9 +IntToStr( config.framesPerCallback));


  { *** set input device properties : *** }
  inputParameters.device:= INPUT_DEVICE;    // FIXME: only "default input device"
  if inputParameters.device = paNoDevice then
    begin
      Memo1.Append( 'Error: No (default) input device available.');
      goto error;
  end;
  inputParameters.channelCount:= config.numInputChannels;
  inputParameters.suggestedLatency:=
    Pa_GetDeviceInfo( inputParameters.device)^.defaultLowInputLatency;
  inputParameters.hostApiSpecificStreamInfo:= NIL;


  { *** set output device properties : *** }
  outputParameters.device:= OUTPUT_DEVICE;    // FIXME: only "default output device"
  if outputParameters.device = paNoDevice then
    begin
      Memo1.Append( 'Error: No (default) output device available.');
      goto error;
    end;
  outputParameters.channelCount:= config.numOutputChannels;
  outputParameters.suggestedLatency:=
    Pa_GetDeviceInfo( outputParameters.device)^.defaultLowOutputLatency;
  outputParameters.hostApiSpecificStreamInfo:= NIL;


  { *** first check if format is supported : *** }
  if config.isInputNotInterleaved
    then inputParameters.sampleFormat:= INPUT_FORMAT {%H-}+ paNonInterleaved
    else inputParameters.sampleFormat:= INPUT_FORMAT;

  if config.isOutputNotInterleaved
    then outputParameters.sampleFormat:= OUTPUT_FORMAT {%H-}+ paNonInterleaved
    else outputParameters.sampleFormat:= OUTPUT_FORMAT;

  err:= Pa_IsFormatSupported( @inputParameters, @outputParameters, SAMPLE_RATE);
  if err = paNoError then
    begin
      { no error - so start test : }
      err:= Pa_OpenStream( stream,
                           @inputParameters,
                           @outputParameters,
                           SAMPLE_RATE,        //FIXME:  fixed sample rate (const)
                           config.framesPerCallback,  //** frames per buffer
                           paClipOff,     //** we won't output out of range samples
                                          //** so don't bother clipping them
                           PPaStreamCallback( @WireCallback),
                           @config);
      if err <> paNoError then goto error;


// HINT:  If You need to get the "STOP-STREAM-MESSAGE"
//        then You can code a "StreamFinishedCallback" function
//        and activate it like this (see TestPASine example):
//
//  Err:= Pa_SetStreamFinishedCallback( Stream,
//     PPaStreamFinishedCallback( @StreamFinished));


  Err:= Pa_StartStream( Stream);
  if Err <> paNoError then goto error;
//  sleep( 200); //instead of asking user if [proceed or terminate ?]
//  sleep( 700); //instead of asking user if [proceed or terminate ?]
  ShowMessage( 'weiter ..');

  Err:= Pa_StopStream( Stream);
  if Err <> paNoError then goto error;

  Err:= Pa_CloseStream( Stream);
  if Err <> paNoError then goto error;
    end
    else Memo1.Append( 'requested format is not supported');

  Exit;

error:
  Result:= err;
end;


{ ***** Buttons ************************************************************** }
procedure TForm1.Button1Click(Sender: TObject);
begin
  close;
end;


procedure TForm1.Button2Click(Sender: TObject);
var err : TPaError;
var c, d, e : dword;
    configIndex   : dword;
    a, b : boolean;
label error;
label done;
begin
  Button2.Enabled:= FALSE;

  configIndex:= 0;
  err:= Pa_Initialize();
  if err <> paNoError then goto error;

  INPUT_DEVICE:=  Pa_GetDefaultInputDevice();
  OUTPUT_DEVICE:= Pa_GetDefaultOutputDevice();

  ShowMessage( 'Please connect audio signal to input and listen for it on output!');
  Edit1.Text:= IntToStr( INPUT_DEVICE);
  Edit2.Text:= IntToStr( OUTPUT_DEVICE);
  Edit3.Text:= IntToStr( INPUT_FORMAT);
  Edit4.Text:= IntToStr( OUTPUT_FORMAT);

  if INPUT_FORMAT = OUTPUT_FORMAT
    then gInOutScaler:= 1.0
    else
      if( (INPUT_FORMAT = paInt16) and (OUTPUT_FORMAT = paFloat32) )
        then gInOutScaler:= 1.0/32768.0
    else
      if( (INPUT_FORMAT = paFloat32) and (OUTPUT_FORMAT = paInt16) )
        then gInOutScaler:= 32768.0;

  { *** Input *** }
  for a:= FALSE to TRUE do
  //for a:= FALSE to FALSE do  // for debug - Interleaved only
  //for a:= TRUE to TRUE do  // for debug - Non-Interleaved only
    begin
      WireConfig.isInputNotInterleaved:= a;
      { *** Output *** }
      for b:= FALSE to TRUE do
      //for b:= FALSE to FALSE do    // for debug - Interleaved only
      //for b:= TRUE to TRUE do    // for debug - Non-Interleaved only
        begin
          WireConfig.isOutputNotInterleaved:= b;
          { *** num Input Channels *** }
          for c:= 1 to 2 do
          //for c:= 1 to 1 do   // for debug
            begin
              WireConfig.numInputChannels:= c;
              { *** num Output Channels *** }
              for d:= 1 to 2 do
              //for d:= 1 to 1 do    // for debug
              //for d:= 2 to 2 do    // for debug
                begin
                  WireConfig.numOutputChannels:= d;
                  // If framesPerCallback = 0 then .. assertion fails in file pa_common/pa_process.c, line 1413: EX. */
                  { *** num Frames in buffer *** }
                  for e:= 1 to 2 do
	          //for e:= 1 to 1 do     // for debug
                  //for e:= 2 to 2 do     // for debug
                    begin
                      WireConfig.framesPerCallback:= e * 64;
                      Memo1.Append( '-----------------------------------------------');
                      Memo1.Append( 'Configuration ' +IntToStr( configIndex));
                      inc( configIndex);
  	              err:= TestConfiguration( WireConfig);
	              { Give user a chance to bail out : }
	              if err = 1
                      // err=1 is never returned - only in the original C code !
                        then
                          begin
	                    err:= paNoError;
	                    goto done;
                          end
    	                else if not(err = paNoError) then goto error;
                    end;
                end;
            end;
        end;
    end;


done:
  // Pa_Terminate() MUST be called before exiting a program which uses PortAudio.
  // Failure to do so may result in serious resource leaks, such as audio devices
  // not being available until the next reboot.
  Pa_Terminate();
  Memo1.Append( '');  //empty line
  Memo1.Append( '** Full duplex sound test complete. **');

  Button2.Enabled:= TRUE;
Exit;

{ proceed this on error only : }
error:
  Pa_Terminate();
  Memo1.Append( '');  //empty line
  Memo1.Append( '** An error occured while using the portaudio stream **');
  Memo1.Append( '     Error number:'+#9 +IntToStr( err));
  Memo1.Append( '     Error message:'+#9 +Pa_GetErrorText( err ));
  //result:= //  return -1;  from original C code ..

  Button2.Enabled:= TRUE;
end;


initialization
  {$I pawire.lrs}


end.

