unit MainUnit; 

{$mode objfpc}{$H+}


// REM:  latest changes 2021-02-10 //


{
  This example is based on the FPC example program  TestPMporttime.pas
  --------------------------------------------------------------------

  PortMidi bindings for FreePascal - Basic PortTime test program
  Latest version available at: http://sourceforge.net/apps/trac/humus/

  Copyright (c) 2010 Roland Schaefer

  PortMidi Portable Real-Time MIDI Library
  Latest version available at: http://sourceforge.net/apps/trac/humus/

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  The text above constitutes the entire PortMidi license; however,
  the PortMusic community also makes the following non-binding requests:

  Any person wishing to distribute modifications to the Software is
  requested to send the modifications to the original developer so that
  they can be incorporated into the canonical version. It is also
  requested that these non-binding requests be included along with the
  license above.
}

interface

uses
  Interfaces,
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, CTypes,
  PortMidi, { using PortMidi here  links the portaudio DLL on BuildTime! }
  PortTime;


type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button4: TButton;
    Button5: TButton;
    ButtonStart: TButton;
    ButtonStop: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    Label1: TLabel;
    Memo1: TMemo;
    Panel1: TPanel;
    Panel2: TPanel;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { private declarations }
    procedure PtTestCallback( timestamp : PtTimestamp; userData : Pointer); cdecl;
  public
    { public declarations }
    TheError : PtError;
  end;

var
  Form1  : TForm1;
  aTime  : CInt32;
  aDummy : CInt32;


implementation

{$R *.lfm}

{ ***** TForm1 Data ********************************************************** }
procedure TForm1.PtTestCallback( timestamp : PtTimestamp; userData : Pointer); cdecl;
begin
  aTime:= timestamp;
  if userData <> NIL then aDummy:= Pt_Time;
end;


procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Edit1.Text:= IntToStr( aTime);
  Edit2.Text:= IntToStr( aDummy);
end;


{ ***** TForm1 *************************************************************** }
procedure TForm1.FormCreate(Sender: TObject);
begin
  ButtonStart.Enabled:= TRUE;
  ButtonStop.Enabled:= FALSE;
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  close;
end;


procedure TForm1.ButtonStartClick(Sender: TObject);
begin
  TheError := Pt_Start( 10, @PtTestCallback, nil);
  Memo1.Append( 'Pt_Start:' +#9 +'error = ' +IntToStr( TheError));
  Timer1.Enabled:= TRUE;

  ButtonStart.Enabled:= FALSE;
  ButtonStop.Enabled:= TRUE;
  Button4.Enabled:= FALSE;
end;


procedure TForm1.ButtonStopClick(Sender: TObject);
begin
  Timer1.Enabled:= FALSE;
  TheError := Pt_Stop();
  Memo1.Append( 'Pt_Stop:' +#9 +'error = ' +IntToStr( TheError));
  Edit1.Text:= '';
  Edit2.Text:= '';

  ButtonStart.Enabled:= TRUE;
  ButtonStop.Enabled:= FALSE;
  Button4.Enabled:= TRUE;
end;


procedure TForm1.Button4Click(Sender: TObject);
begin
  Memo1.Clear;
end;


procedure TForm1.Button5Click(Sender: TObject);
var Info : PPmDeviceInfo;
    i    : integer;
begin
  Memo1.Visible:= FALSE;
  Memo1.Clear;
  //Memo1.Append( 'Num Devices = '   +#9  +IntToStr( Pm_CountDevices));
  //Memo1.Append( 'DefaultInput = '  +#9  +IntToStr( Pm_GetDefaultInputDeviceID));
  //Memo1.Append( 'DefaultOutput = ' +#9  +IntToStr( Pm_GetDefaultOutputDeviceID));

  { enum devices : }
  for i:= 0 to  Pm_CountDevices -1 do
    begin
      Info:= Pm_GetDeviceInfo( i);
      Memo1.Append( 'Device ' +IntToStr(i) +': Interface ' +Info^.interf);
      if i = Pm_GetDefaultOutputDeviceID
        then Memo1.Append( 'Name '      +#9  +Info^.name
                           +'   ( DefaultOutput )')
        else
      if i = Pm_GetDefaultInputDeviceID
        then Memo1.Append( 'Name '      +#9  +Info^.name
                           +'   ( DefaultInput )')
        else Memo1.Append( 'Name '      +#9  +Info^.name);
      if Info^.input > 0
        then Memo1.Append( 'Input '     +#9  +IntToStr( Info^.input));
      if Info^.output > 0
        then Memo1.Append( 'Output '    +#9  +IntToStr( Info^.output));
      Memo1.Append( '');
    end;
  Memo1.Visible:= TRUE;
end;


initialization
//  {$I mainunit.lrs}

end.

